@@ -4,26 +4,27 @@ The core classes provides a means to access them as an ordered collection or as
 """
 __all__ = ["DataFolder", "PlotFolder"]
 
+from Stoner.tools import make_Data
 from .folders.core import baseFolder
-from .folders.mixins import DiskBasedFolder, DataMethodsMixin, PlotMethodsMixin
+
+from .folders.mixins import DiskBasedFolderMixin, DataMethodsMixin, PlotMethodsMixin
 
 
-class DataFolder(DataMethodsMixin, DiskBasedFolder, baseFolder):
+class DataFolder(DataMethodsMixin, DiskBasedFolderMixin, baseFolder):
 
     """Provide an interface to manipulating lots of data files stored within a directory structure on disc.
 
-    By default, the members of the DataFolder are isntances of :class:`Stoner.Data`. The DataFolder emplys a lazy open strategy, so that
-    files are only read in from disc when actually needed.
+    By default, the members of the DataFolder are isntances of :class:`Stoner.Data`. The DataFolder emplys a lazy
+    open strategy, so that files are only read in from disc when actually needed.
 
     .. inheritance-diagram:: DataFolder
 
     """
 
     def __init__(self, *args, **kargs):
-        from Stoner import Data
-
-        self.type = kargs.pop("type", Data)
-        super(DataFolder, self).__init__(*args, **kargs)
+        """Set the default type before creating the DataFolder."""
+        self.type = kargs.pop("type", make_Data(None))
+        super().__init__(*args, **kargs)
 
 
 class PlotFolder(PlotMethodsMixin, DataFolder):
