@@ -9,72 +9,108 @@ It is only necessary to import this module for the subclasses of :py:class:`Ston
 to :py:class:`Stoner.Core.Data`.
 
 """
-__all__ = ["HDF5File", "HDF5Folder", "HGXFile", "SLS_STXMFile", "STXMImage"]
-from .compat import string_types, bytes2str, get_filedialog
-import h5py
-import numpy as _np_
-from .Core import StonerLoadError, metadataObject, DataFile
-from . import Data, DataFolder
-from .Image.core import ImageFile
+__all__ = ["HDF5File", "HDF5Folder", "HGXFile", "SLS_STXMFile", "STXMImage", "HDFFileManager"]
+import importlib
 import os.path as path
 import os
+from copy import deepcopy
+
+import h5py
+import numpy as np
+
+from .compat import string_types, bytes2str, get_filedialog, path_types, str2bytes
+from .Core import StonerLoadError, DataFile
+from .folders import DataFolder
+from .Image.core import ImageFile, ImageArray
+from .core.utils import copy_into
 
 
-def _raise_error(f, message="Not a valid hdf5 file."):
-    """Try to clsoe the filehandle f and raise a StonerLoadError."""
-    try:
-        f.file.close()
-        raise StonerLoadError(message)
-    except Exception:
-        raise StonerLoadError(message)
-
-
-def _open_filename(filename):
-    """Examine a file to see if it is an HDF5 file and open it if so.
+def get_hdf_loader(f, default_loader=lambda *args, **kargs: None):
+    """Look inside the open hdf file for details of what class to use to read this group.
 
     Args:
-        filename (str): Name of the file to open
+        f (h5py.File, h5py.Group):
+            Open hdf file to look for a type attribute that gives the class to use to read this group.
 
     Returns:
-        (f5py.Group): Valid h5py.Group containg data/
-
-    Raises:
-        StonerLoadError if not a valid file.
+        (callable):
+            Callable function that can produce an object of an appropriate class.
     """
-    parts = filename.split(os.pathsep)
-    filename = parts.pop(0)
-    group = ""
-    while len(parts) > 0:
-        if not path.exists(path.join(filename, parts[0])):
-            group = "/".join(parts)
-        else:
-            path.join(filename, parts.pop(0))
+    if "type" not in f.attrs:
+        StonerLoadError("HDF5 Group does not specify the type attribute used to check we can load it.")
+    typ = bytes2str(f.attrs.get("type", ""))
+    if (typ not in globals() or not isinstance(globals()[typ], type)) and "module" not in f.attrs:
+        raise StonerLoadError(
+            "HDF5 Group does not speicify a recongized type and does not specify a module to use to load."
+        )
 
-    with open(filename, "rb") as sniff:  # Some code to manaully look for the HDF5 format magic numbers
-        sniff.seek(0, 2)
-        size = sniff.tell()
-        sniff.seek(0)
-        blk = sniff.read(8)
-        if not blk == b"\x89HDF\r\n\x1a\n":
-            c = 0
-            while sniff.tell() < size and len(blk) == 8:
-                sniff.seek(512 * 2 ** c)
-                c += 1
-                blk = sniff.read(8)
-                if blk == b"\x89HDF\r\n\x1a\n":
-                    break
+    if "module" in f.attrs:
+        mod = importlib.import_module(bytes2str(f.attrs["module"]))
+        cls = getattr(mod, typ)
+        return getattr(cls, "read_hdf5", default_loader)
+    return getattr(globals()[typ], "read_hdf5", default_loader)
+
+
+class HDFFileManager:
+    def __init__(self, filename, mode="r", **kargs):
+        """Initialise context handler.
+
+        Works out the filename and group in cases the input flename includes a path to a sub group.
+
+        Checks the file is actually an h4py file that is openable with the given mode.
+        """
+        self.mode = mode
+        self.handle = None
+        self.file = None
+        self.close = True
+        self.group = ""
+        # Handle the case we're passed an already open h5py object
+        if not isinstance(filename, path_types) or mode == "w":  # passed an already open h5py object
+            self.filename = filename
+            return
+        # Here we deal with a string or path filename
+        parts = str(filename).split(os.path.sep)
+        bits = len(parts)
+        for ix in range(bits):
+            testname = "/".join(parts[: bits - ix])
+            if ix > 0:
+                groupname = "/".join(parts[bits - ix :])
+            if path.exists(testname):
+                filename = testname
+                break
+
+        try:
+            if not mode.startswith("w"):
+                with h5py.File(filename, "r"):
+                    pass
+        except (IOError, OSError):
+            raise StonerLoadError(f"{filename} not at HDF5 File")
+        self.filename = filename
+
+    def __enter__(self):
+        """Open the hdf file with given mode and navigate to the group."""
+        if isinstance(self.filename, (h5py.File, h5py.Group)):  # passed an already open h5py object
+            self.handle = self.filename
+            if isinstance(self.filename, h5py.Group):
+                self.file = self.filename.file
             else:
-                raise StonerLoadError("Couldn't find the HD5 format singature block")
-    try:
-        f = h5py.File(filename, "r+")
-        for grp in group.split("/"):
-            if grp.strip() != "":
-                f = f[grp]
-    except IOError:
-        _raise_error(f, message="Failed to open {} as a n hdf5 file".format(filename))
-    except KeyError:
-        _raise_error(f, message="Could not find group {} in file {}".format(group, filename))
-    return f
+                self.file = self.filename
+            self.close = False
+        elif isinstance(self.filename, path_types):  # Passed something to open
+            handle = h5py.File(self.filename, self.mode)
+            self.file = handle
+            for grp in self.group.split("/"):
+                if grp.strip() != "":
+                    handle = handle[grp]
+            self.handle = handle
+        else:
+            raise StonerLoadError("Note a resource that can be handled with HDF")
+        return self.handle
+
+    def __exit__(self, type, value, traceback):
+        """Ensure we close the hdf file no matter what."""
+        if self.file is not None and self.close:
+            self.file.close()
 
 
 class HDF5File(DataFile):
@@ -82,8 +118,10 @@ class HDF5File(DataFile):
     """A sub class of DataFile that sores itself in a HDF5File or group.
 
     Args:
-        args (tuple): Supplied arguments, only recognises one though !
-        kargs (dict): Dictionary of keyword arguments
+        args (tuple):
+            Supplied arguments, only recognises one though !
+        kargs (dict):
+            Dictionary of keyword arguments
 
     If the first non-keyword arguement is not an h5py File or Group then
     initialises with a blank parent constructor and then loads data, otherwise,
@@ -106,84 +144,108 @@ class HDF5File(DataFile):
     mime_type = ["application/x-hdf"]
 
     def __init__(self, *args, **kargs):
-        """Constructor to catch initialising with an h5py.File or h5py.Group
-           """
+        """Initialise with an h5py.File or h5py.Group."""
         if args and isinstance(args[0], (h5py.File, h5py.Group)):
             args = list(args)
             grp = args.pop(0)
         else:
             grp = None
-        super(HDF5File, self).__init__(*args, **kargs)
+        super().__init__(*args, **kargs)
         if grp is not None:
             self._load(grp, **kargs)
 
     def _load(self, filename, *args, **kargs):
-        """Loads data from a hdf5 file
+        """Load data from a hdf5 file.
 
         Args:
-            h5file (string or h5py.Group): Either a string or an h5py Group object to load data from
+            h5file (string or h5py.Group):
+                Either a string or an h5py Group object to load data from
 
         Returns:
-            itself after having loaded the data
+            self:
+                This object after having loaded the data
         """
         if filename is None or not filename:
             self.get_filename("r")
             filename = self.filename
         else:
             self.filename = filename
-        if isinstance(filename, string_types):  # We got a string, so we'll treat it like a file...
-            f = _open_filename(filename)
-        elif isinstance(filename, h5py.File) or isinstance(filename, h5py.Group):
-            f = filename
-        else:
-            _raise_error(f, message="Couldn't interpret {} as a valid HDF5 file or group or filename".format(filename))
-        if (
-            "type" not in f.attrs or bytes2str(f.attrs["type"]) != "HDF5File"
-        ):  # Ensure that if we have a type attribute it tells us we're the right type !
-            _raise_error(f, message="HDF5 group doesn't hold an HD5File")
-        data = f["data"]
-        if _np_.product(_np_.array(data.shape)) > 0:
-            self.data = data[...]
-        else:
-            self.data = [[]]
-        metadata = f.require_group("metadata")
-        typehints = f.get("typehints", None)
-        if not isinstance(typehints, h5py.Group):
-            typehints = dict()
-        else:
-            typehints = typehints.attrs
-        if "column_headers" in f.attrs:
-            self.column_headers = [x.decode("utf8") for x in f.attrs["column_headers"]]
-            if isinstance(self.column_headers, string_types):
-                self.column_headers = self.metadata.string_to_type(self.column_headers)
-            self.column_headers = [bytes2str(x) for x in self.column_headers]
-        else:
-            raise StonerLoadError("Couldn't work out where my column headers were !")
-        for i in sorted(metadata.attrs):
-            v = metadata.attrs[i]
-            t = typehints.get(i, "Detect")
-            if isinstance(v, string_types) and t != "Detect":  # We have typehints and this looks like it got exported
-                self.metadata["{}{{{}}}".format(i, t).strip()] = "{}".format(v).strip()
+        with HDFFileManager(self.filename, "r") as f:
+            loader = get_hdf_loader(f, default_loader=HDF5File.read_hdf)
+            ret = loader(f, *args, instance=self, **kargs)
+            if isinstance(ret, DataFile):
+                copy_into(ret, self)
+                return self
+            return ret
+
+    @classmethod
+    def read_hdf(cls, filename, *args, **kargs):  # pylint: disable=unused-argument
+        """Create a new HDF5File from an actual HDF file."""
+        self = kargs.pop("instance", cls())
+        if filename is None or not filename:
+            self.get_filename("r")
+            filename = self.filename
+        with HDFFileManager(filename, "r") as f:
+            data = f["data"]
+            if np.product(np.array(data.shape)) > 0:
+                self.data = data[...]
             else:
-                self[i] = metadata.attrs[i]
-        if isinstance(f, h5py.Group):
-            if f.name != "/":
-                self.filename = os.path.join(f.file.filename, f.name)
+                self.data = [[]]
+            metadata = f.require_group("metadata")
+            typehints = f.get("typehints", None)
+            if not isinstance(typehints, h5py.Group):
+                typehints = dict()
             else:
-                self.filename = os.path.realpath(f.file.filename)
-        else:
-            self.filename = os.path.realpath(f.filename)
-        if isinstance(filename, string_types):
-            f.file.close()
+                typehints = typehints.attrs
+            if "column_headers" in f.attrs:
+                self.column_headers = [bytes2str(x) for x in f.attrs["column_headers"]]
+                if isinstance(self.column_headers, string_types):
+                    self.column_headers = self.metadata.string_to_type(self.column_headers)
+                self.column_headers = [bytes2str(x) for x in self.column_headers]
+            else:
+                raise StonerLoadError("Couldn't work out where my column headers were !")
+            for i in sorted(metadata.attrs):
+                v = metadata.attrs[i]
+                t = typehints.get(i, "Detect")
+                if (
+                    isinstance(v, string_types) and t != "Detect"
+                ):  # We have typehints and this looks like it got exported
+                    self.metadata[f"{i}{{{t}}}".strip()] = f"{v}".strip()
+                else:
+                    self[i] = metadata.attrs[i]
+            if isinstance(f, h5py.Group):
+                if f.name != "/":
+                    self.filename = os.path.join(f.file.filename, f.name)
+                else:
+                    self.filename = os.path.realpath(f.file.filename)
+            else:
+                self.filename = os.path.realpath(f.filename)
         return self
 
     def save(self, filename=None, **kargs):
-        """Writes the current object into  an hdf5 file or group within a file in afashion that is compatible with being loaded in again.
+        """Write the current object into  an hdf5 file or group within a file.
+
+        The data is written in afashion that is compatible with being loaded in again.
 
         Args:
-            filename (string or h5py.Group): Either a string, of h5py.File or h5py.Group object into which
-                to save the file. If this is a string, the corresponding file is opened for
-                writing, written to and save again.
+            filename (string or h5py.Group):
+                Either a string, of h5py.File or h5py.Group object into which to save the file. If this is a string,
+                the corresponding file is opened for writing, written to and save again.
+
+        Returns
+            A copy of the object
+        """
+        return self.to_hdf(filename, **kargs)  # Just a pass through to our own to_hdf method
+
+    def to_hdf(self, filename=None, **kargs):  # pylint: disable=unused-argument
+        """Write the current object into  an hdf5 file or group within a file.
+
+        Writes the data in afashion that is compatible with being loaded in again.
+
+        Args:
+            filename (string or h5py.Group):
+                Either a string, of h5py.File or h5py.Group object into which to save the file. If this is a string,
+                the corresponding file is opened for writing, written to and save again.
 
         Returns
             A copy of the object
@@ -193,12 +255,11 @@ class HDF5File(DataFile):
         if filename is None or (isinstance(filename, bool) and not filename):  # now go and ask for one
             filename = self.__file_dialog("w")
             self.filename = filename
-        if isinstance(filename, string_types):
+        if isinstance(filename, path_types):
             mode = "r+" if os.path.exists(filename) else "w"
-            f = h5py.File(filename, mode)
-        elif isinstance(filename, h5py.File) or isinstance(filename, h5py.Group):
-            f = filename
-        try:
+        else:
+            mode = ""
+        with HDFFileManager(filename, mode) as f:
             f.require_dataset(
                 "data",
                 data=self.data,
@@ -213,25 +274,14 @@ class HDF5File(DataFile):
                 try:
                     typehints.attrs[k] = self.metadata._typehints[k]
                     metadata.attrs[k] = self[k]
-                except TypeError:  # We get this for trying to store a bad data type - fallback to metadata export to string
+                except TypeError:
+                    # We get this for trying to store a bad data type - fallback to metadata export to string
                     parts = self.metadata.export(k).split("=")
                     metadata.attrs[k] = "=".join(parts[1:])
-            f.attrs["column_headers"] = [x.encode("utf8") for x in self.column_headers]
+            f.attrs["column_headers"] = [str2bytes(x) for x in self.column_headers]
             f.attrs["filename"] = self.filename
-            f.attrs["type"] = "HDF5File"
-        except Exception as e:
-            if isinstance(filename, str):
-                f.file.close()
-            raise e
-        if isinstance(f, h5py.File):
-            self.filename = f.filename
-        elif isinstance(f, h5py.Group):
-            self.filename = f.file.filename
-        else:
-            self.filename = filename
-        if isinstance(filename, string_types):
-            f.file.close()
-
+            f.attrs["type"] = type(self).__name__
+            f.attrs["module"] = type(self).__module__
         return self
 
 
@@ -248,11 +298,11 @@ class HGXFile(DataFile):
     mime_type = ["application/x-hdf"]
 
     def _load(self, filename, *args, **kargs):
-        """GenX HDF file loader routine.
+        """Load a GenX HDF file.
 
         Args:
-            filename (string or bool): File to load. If None then the existing filename is used,
-                if False, then a file dialog will be used.
+            filename (string or bool):
+                File to load. If None then the existing filename is used, if False, then a file dialog will be used.
 
         Returns:
             A copy of the itself after loading the data.
@@ -262,44 +312,29 @@ class HGXFile(DataFile):
             self.get_filename("r")
         else:
             self.filename = filename
-        with open(filename, "rb") as sniff:  # Some code to manaully look for the HDF5 format magic numbers
-            sniff.seek(0, 2)
-            sniff.seek(0)
-            blk = sniff.read(8)
-            if not blk == b"\x89HDF\r\n\x1a\n":
-                c = 0
-                while len(blk) == 8:
-                    sniff.seek(512 * 2 ** c)
-                    c += 1
-                    blk = sniff.read(8)
-                    if blk == b"\x89HDF\r\n\x1a\n":
-                        break
-                else:
-                    raise StonerLoadError("Couldn't find the HD5 format singature block")
 
-        try:
-            with h5py.File(filename) as f:
+        with HDFFileManager(self.filename, "r") as f:
+            try:
                 if "current" in f and "config" in f["current"]:
                     pass
                 else:
                     raise StonerLoadError("Looks like an unexpected HDF layout!.")
                 self.scan_group(f["current"], "")
                 self.main_data(f["current"]["data"])
-        except IOError:
-            raise StonerLoadError("Looks like an unexpected HDF layout!.")
+            except IOError as err:
+                raise StonerLoadError("Looks like an unexpected HDF layout!.") from err
         return self
 
     def scan_group(self, grp, pth):
         """Recursively list HDF5 Groups."""
         if pth in self.seen:
             return None
-        else:
-            self.seen.append(pth)
+        self.seen.append(pth)
         if not isinstance(grp, h5py.Group):
             return None
         if self.debug:
             if self.debug:
-                print("Scanning in {}".format(pth))
+                print(f"Scanning in {pth}")
         for x in grp:
             if pth == "":
                 new_pth = x
@@ -335,38 +370,41 @@ class HGXFile(DataFile):
             self.column_headers = [str(ix) for ix in self.column_headers]
 
 
-class HDF5FolderMixin(object):
+class HDF5FolderMixin:
 
-    """A mixin class for :py:class:`Stoner.Folders.DataFolder` that provides a method to load and save data from a single HDF5 file with groups.
+    """Provides a method to load and save data from a single HDF5 file with groups.
 
     See :py:class:`Stoner.Folders.DataFolder` for documentation on constructor.
 
-    Datalayout consistns of sub-groups that are either instances of HDF5Files (i.e. have a type attribute that contains 'HDF5File')
-    or are themsleves HDF5Folder instances (with a type attribute that reads 'HDF5Folder').
+    Datalayout consistns of sub-groups that are either instances of HDF5Files (i.e. have a type attribute that
+    contains 'HDF5File') or are themsleves HDF5Folder instances (with a type attribute that reads 'HDF5Folder').
     """
 
     def __init__(self, *args, **kargs):
-        """Constructor for the HDF5Folder Class."""
+        """Initialise the File aatribute."""
         self.File = None
-        super(HDF5FolderMixin, self).__init__(*args, **kargs)
+        super().__init__(*args, **kargs)
 
     def __getter__(self, name, instantiate=True):
-        """Loads the specified name from a file on disk.
+        """Load the specified name from a file on disk.
 
         Parameters:
-            name (key type): The canonical mapping key to get the dataObject. By default
+            name (key type):
+                The canonical mapping key to get the dataObject. By default
                 the baseFolder class uses a :py:class:`regexpDict` to store objects in.
 
         Keyword Arguments:
-            instatiate (bool): IF True (default) then always return a :py:class:`Stoner.Core.Data` object. If False,
+            instatiate (bool):
+                If True (default) then always return a :py:class:`Stoner.Core.Data` object. If False,
                 the __getter__ method may return a key that can be used by it later to actually get the
                 :py:class:`Stoner.Core.Data` object.
 
         Returns:
-            (metadataObject): The metadataObject
+            (metadataObject):
+                The metadataObject
         """
         try:
-            return super(HDF5FolderMixin, self).__getter__(name, instantiate=instantiate)
+            return super().__getter__(name, instantiate=instantiate)
         except (AttributeError, IndexError, KeyError, OSError, IOError) as err:
             if self.debug:
                 print(err)
@@ -382,7 +420,7 @@ class HDF5FolderMixin(object):
         while len(names) > 0:
             next_group = names.pop(0)
             if next_group not in grp:
-                raise IOError("Cannot find {} in {}".format(name, self.File.filename))
+                raise IOError(f"Cannot find {name} in {self.File.filename}")
             grp = grp[next_group]
         tmp = self.loader(grp)
         tmp.filename = grp.name
@@ -395,11 +433,13 @@ class HDF5FolderMixin(object):
         return tmp
 
     def _dialog(self, message="Select Folder", new_directory=True, mode="r+"):
-        """Creates a file dialog box for working with
+        """Create a file dialog box for working with.
 
         Args:
-            message (string): Message to display in dialog
-            new_file (bool): True if allowed to create new directory
+            message (string):
+                Message to display in dialog
+            new_file (bool):
+                True if allowed to create new directory
 
         Returns:
             A directory to be used for the file operation.
@@ -420,17 +460,16 @@ class HDF5FolderMixin(object):
             self.File = h5py.File(self.directory, mode)
             self.File.close()
             return self.directory
-        else:
-            return None
+        return None
 
-    def _visit_func(self, name, obj):
+    def _visit_func(self, name, obj):  # pylint: disable=unused-argument
         """Walker of the HDF5 tree."""
         if isinstance(obj, h5py.Group) and "type" in obj.attrs:
             cls = globals()[obj.attrs["type"]]
             if issubclass(self.loader, cls):
                 self.__setter__(obj.name, obj.name)
             elif obj.attrs["type"] == "HDF5Folder" and self.recursive:
-                self.groups[obj.name.split(path.sep)[-1]] = self.__class__(
+                self.groups[obj.name.split(path.sep)[-1]] = type(self)(
                     obj, pattern=self.pattern, type=self.type, recursive=self.recursive, loader=self.loader
                 )
 
@@ -443,7 +482,7 @@ class HDF5FolderMixin(object):
             raise IOError("HDF5 File not open!")
 
     def getlist(self, recursive=None, directory=None, flatten=False):
-        """Reads the HDF5 File to construct a list of file HDF5File objects"""
+        """Read the HDF5 File to construct a list of file HDF5File objects."""
         if recursive is None:
             recursive = self.recursive
         self.files = []
@@ -457,14 +496,14 @@ class HDF5FolderMixin(object):
                 break
         if directory is None:
             return None
-        if isinstance(directory, string_types):
+        if isinstance(directory, path_types):
             try:
                 self.directory = directory
                 directory = h5py.File(directory, "r+")
                 self.File = directory
                 closeme = True
             except OSError:
-                return super(HDF5FolderMixin, self).getlist(recursive, directory, flatten)
+                return super().getlist(recursive, directory, flatten)
         elif isinstance(directory, h5py.File) or isinstance(directory, h5py.Group):  # Bug out here
             self.File = directory.file
             self.directory = self.File.filename
@@ -479,10 +518,11 @@ class HDF5FolderMixin(object):
         return self
 
     def save(self, root=None):
-        """Saves a load of files to a single HDF5 file, creating groups as it goes.
+        """Save a load of files to a single HDF5 file, creating groups as it goes.
 
         Keyword Arguments:
-            root (string): The name of the HDF5 file to save to if set to None, will prompt for a filename.
+            root (string):
+                The name of the HDF5 file to save to if set to None, will prompt for a filename.
 
         Return:
             A list of group paths in the HDF5 file
@@ -491,16 +531,16 @@ class HDF5FolderMixin(object):
         if root is None and isinstance(self.File, h5py.File):
             root = self.File
         elif root is None and not isinstance(self.File, h5py.File):
-            root = h5py.File(self.directory)
+            root = h5py.File(self.directory, mode="w")
             self.File = root
             closeme = True
         if root is None or (isinstance(root, bool) and not root):
             # now go and ask for one
             root = self._dialog()
-            root = h5py.File(root)
+            root = h5py.File(root, mode="a")
             self.File = root
             closeme = True
-        if isinstance(root, string_types):
+        if isinstance(root, path_types):
             mode = "r+" if path.exists(root) else "w"
             root = h5py.File(root, mode)
             self.File = root
@@ -510,7 +550,7 @@ class HDF5FolderMixin(object):
         # root should be an open h5py file
         root.attrs["type"] = "HDF5Folder"
         for ix, obj in enumerate(self):
-            name = os.path.basename(getattr(obj, "filename", "obj-{}".format(ix)))
+            name = os.path.basename(getattr(obj, "filename", f"obj-{ix}"))
             if name not in root:
                 root.create_group(name)
             name = root[name]
@@ -528,16 +568,17 @@ class HDF5FolderMixin(object):
 
 class HDF5Folder(HDF5FolderMixin, DataFolder):
 
-    """Just enforces the loader attriobute to be an HDF5File!"""
+    """Just enforces the loader attriobute to be an HDF5File."""
 
     def __init__(self, *args, **kargs):
+        """Ensure the loader routine is set for HDF5Files."""
         self.loader = HDF5File
-        super(HDF5Folder, self).__init__(*args, **kargs)
+        super().__init__(*args, **kargs)
 
 
 class SLS_STXMFile(DataFile):
 
-    """Load images from the Swiss Light Source Pollux beamline"""
+    """Load images from the Swiss Light Source Pollux beamline."""
 
     priority = 16
     compression = "gzip"
@@ -546,10 +587,11 @@ class SLS_STXMFile(DataFile):
     mime_type = ["application/x-hdf"]
 
     def _load(self, filename, *args, **kargs):
-        """Loads data from a hdf5 file
+        """Load data from the hdf5 file produced by Pollux.
 
         Args:
-            h5file (string or h5py.Group): Either a string or an h5py Group object to load data from
+            h5file (string or h5py.Group):
+                Either a string or an h5py Group object to load data from
 
         Returns:
             itself after having loaded the data
@@ -559,15 +601,15 @@ class SLS_STXMFile(DataFile):
             filename = self.filename
         else:
             self.filename = filename
-        if isinstance(filename, string_types):  # We got a string, so we'll treat it like a file...
+        if isinstance(filename, path_types):  # We got a string, so we'll treat it like a file...
             try:
                 f = h5py.File(filename, "r+")
-            except IOError:
-                raise StonerLoadError("Failed to open {} as a n hdf5 file".format(filename))
+            except IOError as err:
+                raise StonerLoadError(f"Failed to open {filename} as a n hdf5 file") from err
         elif isinstance(filename, h5py.File) or isinstance(filename, h5py.Group):
             f = filename
         else:
-            raise StonerLoadError("Couldn't interpret {} as a valid HDF5 file or group or filename".format(filename))
+            raise StonerLoadError(f"Couldn't interpret {filename} as a valid HDF5 file or group or filename")
         items = [x for x in f.items()]
         if len(items) == 1 and items[0][0] == "entry1":
             group1 = [x for x in f["entry1"]]
@@ -579,7 +621,7 @@ class SLS_STXMFile(DataFile):
             raise StonerLoadError("HDF5 file lacks single top level group called entry1")
         root = f["entry1"]
         data = root["counter0"]["data"]
-        if _np_.product(_np_.array(data.shape)) > 0:
+        if np.product(np.array(data.shape)) > 0:
             self.data = data[...]
         else:
             self.data = [[]]
@@ -591,7 +633,7 @@ class SLS_STXMFile(DataFile):
         else:
             self["original filename"] = f.file.filename
 
-        if isinstance(filename, string_types):
+        if isinstance(filename, path_types):
             f.file.close()
         self["Loaded from"] = self.filename
 
@@ -599,10 +641,13 @@ class SLS_STXMFile(DataFile):
             self.metadata["actual_x"] = self.metadata["instrument.sample_x.data"].reshape(self.shape)
         if "instrument.sample_y.data" in self.metadata:
             self.metadata["actual_y"] = self.metadata["instrument.sample_y.data"].reshape(self.shape)
-        self.metadata["sample_x"], self.metadata["sample_y"] = _np_.meshgrid(
+        self.metadata["sample_x"], self.metadata["sample_y"] = np.meshgrid(
             self.metadata["counter0.sample_x"], self.metadata["counter0.sample_y"]
         )
-
+        if "control.data" in self.metadata:
+            self.metadata["beam current"] = ImageArray(self.metadata["control.data"].reshape(self.data.shape))
+            self.metadata["beam current"].metadata = self.metadata
+        self.data = self.data[::-1]
         return self
 
     def scan_meta(self, group):
@@ -614,54 +659,75 @@ class SLS_STXMFile(DataFile):
             if isinstance(thing, h5py.Group):
                 self.scan_meta(thing)
             elif isinstance(thing, h5py.Dataset):
-                if len(thing.shape) > 1:
+                if thing.ndim > 1:
                     continue
-                if _np_.product(thing.shape) == 1:
+                if np.product(thing.shape) == 1:
                     self.metadata[name] = thing[0]
                 else:
                     self.metadata[name] = thing[...]
         for attr in group.attrs:
-            self.metadata["{}.{}".format(root, attr)] = group.attrs[attr]
+            self.metadata[f"{root}.{attr}"] = group.attrs[attr]
 
 
 class STXMImage(ImageFile):
 
-    """An instance of KerrArray that will load itself from a Swiss Light Source STXM image"""
+    """An instance of KerrArray that will load itself from a Swiss Light Source STXM image."""
+
+    # pylint: disable=no-member
 
     _reduce_metadata = False
 
+    _patterns = ["*.hdf5", "*.hdf"]
+    mime_type = ["application/x-hdf"]
+    priority = 16
+
     def __init__(self, *args, **kargs):
-        """Construct a STXMImage file.
+        """Initialise and load a STXM image produced by Pollux.
 
         Keyword Args:
-            regrid (bool): If set True, the gridimage() method is automatically called to re-grid the image to known co-ordinates."""
+            regrid (bool):
+                If set True, the gridimage() method is automatically called to re-grid the image to known co-ordinates.
+        """
         regrid = kargs.pop("regrid", False)
-        if len(args) > 0 and isinstance(args[0], string_types):
+        bcn = kargs.pop("bcn", False)
+        if len(args) > 0 and isinstance(args[0], path_types):
             d = SLS_STXMFile(args[0])
             args = args[1:]
+            super().__init__(*args, **kargs)
+            self.image = d.data
+            self.metadata = deepcopy(d.metadata)
+            self.filename = d.filename
+        elif len(args) > 0 and isinstance(args[0], ImageFile):
+            src = args[0]
+            args = args[1:]
+            super().__init__(src.image, *args, **kargs)
+        elif len(args) > 0 and isinstance(args[0], ImageArray):
+            src = args[0]
+            args = args[1:]
+            super().__init__(src, *args, **kargs)
         else:
-            d = Data()
-        super(STXMImage, self).__init__(*args[1:], **kargs)
-        self.image = d.data
-        self.metadata.update(d.metadata)
-        self.filename = d.filename
+            super().__init__(*args, **kargs)
         if isinstance(regrid, tuple):
             self.gridimage(*regrid)
         elif isinstance(regrid, dict):
             self.gridimage(**regrid)
         elif regrid:
             self.gridimage()
+        if bcn:
+            if regrid:
+                self.metadata["beam current"] = self.metadata["beam current"].gridimage()
+            self.image /= self["beam current"]
 
-    def __floordiv__(self, other):
-        if isinstance(other, metadataObject):
-            if self["collection.polarization.value"] > 0 and other["collection.polarization.value"] < 0:
-                plus, minus = self, other
-            elif self["collection.polarization.value"] < 0 and other["collection.polarization.value"] > 0:
-                plus, minus = other, self
-            else:
-                raise ValueError("XMCD Ratio can only be found from a positive and minus image")
-            ret = self.clone
-            ret.image = (plus.image - minus.image) / (plus.image + minus.image)
-            return ret
-        else:
-            raise TypeError("Can only do XMCD calculation with another STXMFile")
+    @property
+    def polarization(self):
+        """Return the sign of the polarization used for the image.
+
+        Returns:
+            +1,-1 or 0 for positive, negative or unpolariused light used for the image.
+        """
+        return np.sign(self.get("collection.polarization.value", 0))
+
+    def _load(self, filename, *args, **kargs):
+        """Pass through to SLS_STXMFile._load."""
+        self.__init__(filename, *args, **kargs)
+        return self
