@@ -1,6 +1,5 @@
 # -*- coding: utf-8 -*-
-"""
-Stoner.Zip module - sipport reading DataFile like objects into and outof standard zip files.
+"""Support reading DataFile like objects into and outof standard zip files.
 
 Classes Include
 
@@ -13,41 +12,45 @@ import os.path as path
 from traceback import format_exc
 import fnmatch
 
-from .compat import string_types, bytes2str, str2bytes, get_filedialog, _pattern_type
+from .compat import string_types, str2bytes, get_filedialog, _pattern_type, path_types
 from .Core import DataFile, StonerLoadError
-from .Folders import DiskBasedFolder
+from .Folders import DiskBasedFolderMixin
 from .folders.core import baseFolder
 from .folders.utils import pathjoin
+from .tools import copy_into, make_Data
 
 
 def test_is_zip(filename, member=""):
     """Recursively searches for a zipfile in the tree.
 
     Args:
-        filename (str): Path to test whether it is a zip file or not.
+        filename (str):
+            Path to test whether it is a zip file or not.
 
     Keyword Arguments:
-        member (str): Used in recursive calls to identify the path within the zip file
+        member (str):
+            Used in recursive calls to identify the path within the zip file
 
     Returns:
-        False or (filename,member): Returns False if not a zip file, otherwise the actual filename of the zip file and the nanme of the member within that
+        False or (filename,member):
+            Returns False if not a zip file, otherwise the actual filename of the zip file and the nanme of the
+            member within that
         zipfile.
     """
-    if not filename or filename == "":
+    if not filename or str(filename) == "":
         return False
-    elif zf.is_zipfile(filename):
+    if zf.is_zipfile(filename):
         return filename, member
+    part = path.basename(filename)
+    newfile = path.dirname(filename)
+    if newfile == filename:  # reached the end of the line
+        part = filename
+        newfile = ""
+    if member != "":
+        newmember = path.join(part, member)
     else:
-        part = path.basename(filename)
-        newfile = path.dirname(filename)
-        if newfile == filename:  # reached the end of the line
-            part = filename
-            newfile = ""
-        if member != "":
-            newmember = path.join(part, member)
-        else:
-            newmember = part
-        return test_is_zip(newfile, newmember)
+        newmember = part
+    return test_is_zip(newfile, newmember)
 
 
 class ZippedFile(DataFile):
@@ -66,7 +69,7 @@ class ZippedFile(DataFile):
     mime_type = ["application/zip"]
 
     def __init__(self, *args, **kargs):
-        "Constructor to catch initialising with an open zf.ZipFile"
+        """Catch initialising with an open zf.ZipFile."""
         if len(args) > 0:
             other = args[0]
             if isinstance(other, zf.ZipFile):
@@ -81,11 +84,11 @@ class ZippedFile(DataFile):
                 ):
                     kargs["filename"] = kargs["filename"][2:]
                 if kargs["filename"] not in other.namelist():  # New file not in the zip file yet
-                    raise StonerLoadError("File {} not found in zip file {}".format(kargs["filename"], other.filename))
+                    raise StonerLoadError(f"File {kargs['filename']} not found in zip file {other.filename}")
                 # Ok, by this point we have a zipfile which has a file in it. Construct ourselves and then load
-                super(ZippedFile, self).__init__(**kargs)
+                super().__init__(**kargs)
                 self._extract(other, kargs["filename"])
-            elif isinstance(other, string_types):  # Passed a string - so try as a zipfile
+            elif isinstance(other, path_types):  # Passed a string - so try as a zipfile
                 if zf.is_zipfile(other):
                     other = zf.ZipFile(other, "a")
                     args = args = list(args)
@@ -94,27 +97,32 @@ class ZippedFile(DataFile):
                     args = test_is_zip(other)
                 self.__init__(*args, **kargs)
             else:
-                super(ZippedFile, self).__init__(*args, **kargs)
+                super().__init__(*args, **kargs)
+        else:
+            super().__init__(*args, **kargs)
 
     def _extract(self, archive, member):
         """Responsible for actually reading the zip file archive.
 
         Args:
-            archive (zipfile.ZipFile): An open zip archive
-            member (string): The name of one member of the zip file
+            archive (zipfile.ZipFile):
+                An open zip archive
+            member (string):
+                The name of one member of the zip file
 
         Return:
             A datafile like instance
         """
-        tmp = DataFile()
         info = archive.getinfo(member)
-        data = bytes2str(archive.read(info))  # In Python 3 this would be a bytes
-        self.__init__(tmp << data)
+        data = archive.read(info)  # In Python 3 this would be a bytes
+        tmp = make_Data(data)
+        copy_into(tmp, self)
+        # self.__init__(tmp << data)
         self.filename = path.join(archive.filename, member)
         return self
 
     def _load(self, filename=None, *args, **kargs):
-        "Load a file from the zip file, openining it as necessary"
+        """Load a file from the zip file, openining it as necessary."""
         if filename is None or not filename:
             self.get_filename("r")
         else:
@@ -129,31 +137,24 @@ class ZippedFile(DataFile):
                     close_me = False
                 member = kargs.get("member", other.namelist()[0])
                 solo_file = len(other.namelist()) == 1
-            elif isinstance(self.filename, string_types) and zf.is_zipfile(
+            elif isinstance(self.filename, path_types) and zf.is_zipfile(
                 self.filename
             ):  # filename is a string that is a zip file
                 other = zf.ZipFile(self.filename, "a")
                 member = kargs.get("member", other.namelist()[0])
                 close_me = True
                 solo_file = len(other.namelist()) == 1
-            elif isinstance(self.filename, string_types) and test_is_zip(
-                self.filename
-            ):  # Filename is something buried in a zipfile
-                other, member = test_is_zip(other)
-                other = zf.ZipFile(other, "r")
-                close_me = True
-                solo_file = len(other.namelist()) == 1
             else:
-                raise StonerLoadError("{} does  not appear to be a real zip file".format(self.filename))
-        except StonerLoadError as e:
-            raise e
-        except Exception:
+                raise StonerLoadError(f"{self.filename} does  not appear to be a real zip file")
+        except StonerLoadError:
+            raise
+        except Exception as err:  # pylint: disable=W0703 # Catching everything else here
             try:
                 exc = format_exc()
                 other.close()
             except (AttributeError, NameError, ValueError, TypeError, zf.BadZipFile, zf.LargeZipFile):
                 pass
-            raise StonerLoadError("{} threw an error when opening\n{}".format(self.filename, exc))
+            raise StonerLoadError(f"{self.filename} threw an error when opening\n{exc}") from err
         # Ok we can try reading now
         self._extract(other, member)
         if close_me:
@@ -163,10 +164,11 @@ class ZippedFile(DataFile):
         return self
 
     def save(self, filename=None, **kargs):
-        """Overrides the save method to allow ZippedFile to be written out to disc (as a mininmalist output)
+        """Override the save method to allow ZippedFile to be written out to disc (as a mininmalist output).
 
         Args:
-            filename (string or zipfile.ZipFile instance): Filename to save as (using the same rules as for the load routines)
+            filename (string or zipfile.ZipFile instance):
+                Filename to save as (using the same rules as for the load routines)
 
         Returns:
             A copy of itself.
@@ -177,20 +179,20 @@ class ZippedFile(DataFile):
             filename = self.__file_dialog("w")
         compression = kargs.pop("compression", zf.ZIP_DEFLATED)
         try:
-            if isinstance(filename, string_types):  # We;ve got a string filename
+            if isinstance(filename, path_types):  # We;ve got a string filename
                 if test_is_zip(filename):  # We can find an existing zip file somewhere in the filename
                     zipfile, member = test_is_zip(filename)
                     zipfile = zf.ZipFile(zipfile, "a")
                     close_me = True
                 elif path.exists(filename):  # The fiule exists but isn't a zip file
-                    raise IOError("{} Should either be a zip file or a new zip file".format(filename))
+                    raise IOError(f"{filename} Should either be a zip file or a new zip file")
                 else:  # Path doesn't exist, use extension of file part to find where the zip file should be
                     parts = path.split(filename)
                     for i, part in enumerate(parts):
                         if path.splitext(part)[1].lower() == ".zip":
                             break
                     else:
-                        raise IOError("Can't figure out where the zip file is in {}".format(filename))
+                        raise IOError(f"Can't figure out where the zip file is in {filename}")
                     zipfile = zf.ZipFile(path.join(*parts[: i + 1]), "w", compression, True)
                     close_me = True
                     member = path.join("/", *parts[i + 1 :])
@@ -216,19 +218,18 @@ class ZippedFile(DataFile):
             zipfile.writestr(member, str2bytes(str(self)))
             if close_me:
                 zipfile.close()
-        except Exception:
+        except (zipfile.BadZipFile, IOError, TypeError, ValueError) as err:
             error = format_exc()
             try:
                 zipfile.close()
-            except Exception:
-                raise IOError("Error saving zipfile\n{}".format(error))
-            raise IOError("Error saving zipfile\n{}".format(error))
+            finally:
+                raise IOError(f"Error saving zipfile\n{error}") from err
         return self
 
 
-class ZipFolderMixin(object):
+class ZipFolderMixin:
 
-    """A sub class of :py:class:`Stoner.Folders.DataFolder` that provides a method to load and save data from a single Zip file.
+    """Provides methods to load and save data from a single Zip file.
 
     See :py:class:`Stoner.Folders.DataFolder` for documentation on constructor.
 
@@ -248,14 +249,16 @@ class ZipFolderMixin(object):
     }
 
     def __init__(self, *args, **kargs):
-        "Constructor for the ZipFolderMixin Class."
+        """Initialise the file attribute."""
         self.File = None
-        super(ZipFolderMixin, self).__init__(*args, **kargs)
+        super().__init__(*args, **kargs)
 
     @property
     def full_key(self):
         """Generate a full canonical path through the zip archive to this file."""
-        return path.relpath(self.path, self.File.filename).replace(path.sep, "/")
+        filename = path.splitdrive(self.File.filepath)[1]
+        name = path.splitdrive(self.path)[1]
+        return path.relpath(name, filename).replace(path.sep, "/")
 
     @property
     def key(self):
@@ -267,12 +270,12 @@ class ZipFolderMixin(object):
         """Set the immediate filename that will be used when the file is saved."""
         self.path = pathjoin(self.directory, value)
 
-    def _dialog(self, message="Select Folder", new_directory=True, mode="r"):
-        """Creates a file dialog box for working with
+    def _dialog(self, mode="r"):
+        """Create a file dialog box for working with.
 
         Args:
-            message (string): Message to display in dialog
-            new_file (bool): True if allowed to create new directory
+            mode (string):
+                Where the dialog is for opening or saving the file.
 
         Returns:
             A directory to be used for the file operation.
@@ -290,11 +293,10 @@ class ZipFolderMixin(object):
             self.File = zf.ZipFile(self.directory, mode)
             self.File.close()
             return self.directory
-        else:
-            return None
+        return None
 
     def getlist(self, recursive=None, directory=None, flatten=None):
-        "Reads the Zip File to construct a list of ZipFile objects"
+        """Read the Zip File to construct a list of ZipFile objects."""
         if recursive is None:
             recursive = self.recursive
         self.files = []
@@ -313,9 +315,9 @@ class ZipFolderMixin(object):
             else:
                 self.File = zf.ZipFile(directory, "r")
                 close_me = True
-        elif isinstance(directory, string_types) and path.isdir(directory):  # Fall back to DataFolder
-            return super(ZipFolderMixin, self).getlist(recursive, directory, flatten)
-        elif isinstance(directory, string_types) and zf.is_zipfile(directory):
+        elif isinstance(directory, path_types) and path.isdir(directory):  # Fall back to DataFolder
+            return super().getlist(recursive, directory, flatten)
+        elif isinstance(directory, path_types) and zf.is_zipfile(directory):
             self.File = zf.ZipFile(directory, "r")
             close_me = True
         elif isinstance(self.File, zf.ZipFile):
@@ -325,7 +327,7 @@ class ZipFolderMixin(object):
                 self.File = zf.ZipFile(self.File.filename, "r")
                 close_me = True
         else:
-            raise IOError("{} does not appear to be zip file!".format(directory))
+            raise IOError(f"{directory} does not appear to be zip file!")
         # At this point directory contains an open h5py.File object, or possibly a group
         self.path = self.File.filename
         files = [x.filename for x in self.File.filelist]
@@ -376,28 +378,31 @@ class ZipFolderMixin(object):
     def __clone__(self, other=None, attrs_only=False):
         """Do whatever is necessary to copy attributes from self to other."""
         if other is None and attrs_only:
-            other = self.__class__(readlist=False)
+            other = type(self)(readlist=False)
         for arg in self._defaults:
             if hasattr(self, arg):
                 setattr(other, arg, getattr(self, arg))
-        return super(ZipFolderMixin, self).__clone__(other=other, attrs_only=attrs_only)
+        return super().__clone__(other=other, attrs_only=attrs_only)
 
     def __getter__(self, name, instantiate=True):
-        """Loads the specified name from a compressed archive.
+        """Load the specified name from a compressed archive.
 
         Parameters:
-            name (key type): The canonical mapping key to construct the path from.
+            name (key type):
+                The canonical mapping key to construct the path from.
 
         Keyword Arguments:
-            instatiate (bool): IF True (default) then always return a :py:class:`Stoner.Core.Data` object. If False,
+            instatiate (bool):
+                IF True (default) then always return a :py:class:`Stoner.Core.Data` object. If False,
                 the __getter__ method may return a key that can be used by it later to actually get the
                 :py:class:`Stoner.Core.Data` object.
 
         Returns:
-            (metadataObject): The metadataObject
+            (metadataObject):
+                The metadataObject
         """
         try:
-            return super(ZipFolderMixin, self).__getter__(name, instantiate=instantiate)
+            return super().__getter__(name, instantiate=instantiate)
         except (AttributeError, IndexError, KeyError, OSError, IOError) as err:
             if self.debug:
                 print(err)
@@ -418,7 +423,8 @@ class ZipFolderMixin(object):
         """Look for a given name in the ZipFolder namelist.
 
         Parameters:
-            name(str): Name of an object
+            name(str):
+                Name of an object
 
         Returns:
             A canonical key name for that file
@@ -443,13 +449,14 @@ class ZipFolderMixin(object):
                     return pth
         except AttributeError:
             pass
-        return super(ZipFolderMixin, self).__lookup__(name)
+        return super().__lookup__(name)
 
     def save(self, root=None):
-        """Saves a load of files to a single Zip file, creating members as it goes.
+        """Save a load of files to a single Zip file, creating members as it goes.
 
         Keyword Arguments:
-            root (string): The name of the Zip file to save to if set to None, will prompt for a filename.
+            root (string):
+                The name of the Zip file to save to if set to None, will prompt for a filename.
 
         Return:
             A list of group paths in the Zip file
@@ -470,8 +477,10 @@ class ZipFolderMixin(object):
         """Create a virtual path of groups in the Zip file and save data.
 
         Args:
-            f(DataFile):  A DataFile instance to save
-            trail (list): The trail of groups
+            f(DataFile):
+                A DataFile instance to save
+            trail (list):
+                The trail of groups
 
         Returns:
             The new filename of the saved DataFile.
@@ -498,7 +507,7 @@ class ZipFolderMixin(object):
         return f.filename
 
 
-class ZipFolder(ZipFolderMixin, DiskBasedFolder, baseFolder):
+class ZipFolder(ZipFolderMixin, DiskBasedFolderMixin, baseFolder):
 
     """A sub class of DataFile that sores itself in a zip file.
 
@@ -507,7 +516,3 @@ class ZipFolder(ZipFolderMixin, DiskBasedFolder, baseFolder):
     calls parent constructor.
 
     """
-
-    def __init__(self, *args, **kargs):
-        "Constructor for the ZipFolderMixin Class."
-        super(ZipFolder, self).__init__(*args, **kargs)
