@@ -1,23 +1,28 @@
 """Example of nDimArrhenius Fit."""
-from Stoner import Data
-import Stoner.Fit as SF
+# pylint: disable=invalid-name
 from numpy import linspace, ones_like
 from numpy.random import normal
 
+from Stoner import Data
+from Stoner.analysis.fitting.models.tunnelling import (
+    fowlerNordheim,
+    FowlerNordheim,
+)
+
 # Make some data
 V = linspace(-4, 4, 1000)
-I = SF.fowlerNordheim(V, 2500, 3.2, 15.0) + normal(size=len(V), scale=1e-6)
+I = fowlerNordheim(V, 2500, 3.2, 15.0) + normal(size=len(V), scale=1e-6)
 dI = ones_like(V) * 10e-6
 
 d = Data(V, I, dI, setas="xye", column_headers=["Bias", "Current", "Noise"])
 
 d.curve_fit(
-    SF.fowlerNordheim, p0=[2500, 3.2, 15.0], result=True, header="curve_fit"
+    fowlerNordheim, p0=[2500, 3.2, 15.0], result=True, header="curve_fit"
 )
 d.setas = "xyey"
 d.plot(fmt=["r.", "b-"])
 d.annotate_fit(
-    SF.fowlerNordheim,
+    fowlerNordheim,
     x=0.2,
     y=0.6,
     prefix="fowlerNordheim",
@@ -25,7 +30,7 @@ d.annotate_fit(
 )
 
 d.setas = "xye"
-fit = SF.FowlerNordheim()
+fit = FowlerNordheim()
 p0 = [2500, 5.2, 15.0]
 p0 = fit.guess(I, x=V)
 for p, v, mi, mx in zip(
@@ -33,7 +38,7 @@ for p, v, mi, mx in zip(
 ):
     p0[p].value = v
     p0[p].bounds = [mi, mx]
-d.lmfit(SF.FowlerNordheim, p0=p0, result=True, header="lmfit")
+d.lmfit(FowlerNordheim, p0=p0, result=True, header="lmfit")
 d.setas = "x...y"
 d.plot(fmt="g-")
 d.annotate_fit(
