@@ -1,23 +1,25 @@
 """Example of nDimArrhenius Fit."""
-from Stoner import Data
-import Stoner.Fit as SF
+# pylint: disable=invalid-name
 from numpy import linspace, ones_like
 from numpy.random import normal
 
+from Stoner import Data
+from Stoner.analysis.fitting.models.tunnelling import simmons, Simmons
+
 # Make some data
 V = linspace(-4, 4, 101)
-I = SF.simmons(V, 2500, 3.2, 15.0) + normal(size=len(V), scale=5e-7)
+I = simmons(V, 2500, 3.2, 15.0) + normal(size=len(V), scale=5e-7)
 dI = ones_like(V) * 500e-9
 
 p0 = p0 = [2500, 3, 10.0]
 
 d = Data(V, I, dI, setas="xye", column_headers=["Bias", "Current", "Noise"])
 
-d.curve_fit(SF.simmons, p0=p0, result=True, header="curve_fit", maxfev=2000)
+d.curve_fit(simmons, p0=p0, result=True, header="curve_fit", maxfev=2000)
 d.setas = "xyey"
 d.plot(fmt=["r,", "b-"], capsize=1)
 d.annotate_fit(
-    SF.simmons,
+    simmons,
     x=0.25,
     y=0.25,
     prefix="simmons",
@@ -25,8 +27,8 @@ d.annotate_fit(
 )
 
 d.setas = "xye"
-fit = SF.Simmons()
-d.lmfit(SF.Simmons, p0=p0, result=True, header="lmfit", maxfev=2000)
+fit = Simmons()
+d.lmfit(Simmons, p0=p0, result=True, header="lmfit", maxfev=2000)
 d.setas = "x...y"
 d.plot(fmt="g-", label="lmfit")
 d.annotate_fit(
