@@ -1,12 +1,17 @@
 """Test Weak-localisation fitting."""
-from Stoner import Data
-import Stoner.Fit as SF
-from numpy import linspace, ones_like
+# pylint: disable=invalid-name
+from numpy import logspace, ones_like, log10
 from numpy.random import normal
 
-B = linspace(2, 100, 26)
+from Stoner import Data
+from Stoner.analysis.fitting.models.e_transport import (
+    fluchsSondheimer,
+    FluchsSondheimer,
+)
+
+B = logspace(log10(2), 2, 26)
 params = [12.5, 0.75, 1e3]
-G = SF.fluchsSondheimer(B, *params) + normal(size=len(B), scale=5e-5)
+G = fluchsSondheimer(B, *params) + normal(size=len(B), scale=5e-5)
 dG = ones_like(B) * 5e-5
 d = Data(
     B,
@@ -16,22 +21,22 @@ d = Data(
     column_headers=["Thickness (nm)", "Conductance", "dConductance"],
 )
 
-d.curve_fit(SF.fluchsSondheimer, p0=params, result=True, header="curve_fit")
+d.curve_fit(fluchsSondheimer, p0=params, result=True, header="curve_fit")
 
 d.setas = "xye"
-d.lmfit(SF.FluchsSondheimer, p0=params, result=True, header="lmfit")
+d.lmfit(FluchsSondheimer, result=True, header="lmfit")
 
 d.setas = "xyeyy"
 d.plot(fmt=["r.", "b-", "g-"])
 
 d.annotate_fit(
-    SF.fluchsSondheimer,
+    fluchsSondheimer,
     x=0.2,
     y=0.6,
     fontdict={"size": "x-small", "color": "blue"},
 )
 d.annotate_fit(
-    SF.FluchsSondheimer,
+    FluchsSondheimer,
     x=0.2,
     y=0.4,
     fontdict={"size": "x-small", "color": "green"},
