@@ -1,15 +1,14 @@
-"""Demo of new Stoner.Analysis.AnalyseFile.lmfit
-"""
-from __future__ import print_function
-from Stoner import __home__
+"""Demo of new Stoner.Analysis.AnalyseFile.lmfit."""
+# pylint: disable=invalid-name
 from os.path import join
-from Stoner.Fit import cfg_data_from_ini, cfg_model_from_ini
+
+from Stoner.analysis.fitting.models import cfg_data_from_ini, cfg_model_from_ini
+from Stoner import __home__
 
 config = join(__home__, "..", "scripts", "PCAR-New.ini")
 datafile = join(__home__, "..", "sample-data", "PCAR Co Data.csv")
 
 d = cfg_data_from_ini(config, datafile)
-d.x += 0.244
 model, p0 = cfg_model_from_ini(config, data=d)
 
 fit = d.lmfit(model, result=True, header="lmfit", output="report")
