@@ -1,7 +1,9 @@
-"""Example plot using experimental GBStyle"""
+"""Example plot using experimental GBStyle."""
+# pylint: disable=invalid-name
+import os.path as path
+
 from Stoner import Data, __home__
 from Stoner.plot.formats import GBPlotStyle
-import os.path as path
 
 filename = path.realpath(
     path.join(__home__, "..", "doc", "samples", "sample.txt")
